@@ -914,14 +914,14 @@ do_mount (GVfsBackend *backend,
   op_backend->volume_symbolic_icon = g_vfs_get_volume_symbolic_icon (device);
   g_object_unref (device);
 
+  g_signal_connect (op_backend->gudev_client, "uevent", G_CALLBACK (on_uevent), op_backend);
+
+  op_backend->file_cache = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
   LIBMTP_Init ();
 
   get_device (backend, host, G_VFS_JOB (job));
   if (!G_VFS_JOB (job)->failed) {
-    g_signal_connect (op_backend->gudev_client, "uevent", G_CALLBACK (on_uevent), op_backend);
-
-    op_backend->file_cache = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-
     GMountSpec *mtp_mount_spec = g_mount_spec_new ("mtp");
     g_mount_spec_set (mtp_mount_spec, "host", host);
     g_vfs_backend_set_mount_spec (backend, mtp_mount_spec);
@@ -2022,18 +2022,6 @@ do_push (GVfsBackend *backend,
     goto exit;
   }
 
-  /*
-   * Don't do this for >=4GB files with android extensions. Some devices
-   * have trouble transferring large files this way. eg: Nexus 5.
-   */
-  if (g_file_info_get_size (info) > G_MAXUINT32 &&
-      G_VFS_BACKEND_MTP (backend)->android_extension) {
-    g_vfs_job_failed_literal (G_VFS_JOB (job),
-                              G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                              _("Operation unsupported"));
-    goto exit;
-  }
-
   gboolean source_is_dir =
     g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY;
 
@@ -2456,21 +2444,6 @@ do_read (GVfsBackend *backend,
       goto exit;
     }
 
-    /*
-     * Almost all android devices have a bug where they do not enforce
-     * POSIX semantics for read past EOF, leading to undefined
-     * behaviour including device-side hangs. We'd better handle it
-     * here.
-     */
-    if (offset >= handle->size) {
-      g_debug ("(II) skipping read with offset past EOF\n");
-      actual = 0;
-      goto finished;
-    } else if (offset + bytes_requested > handle->size) {
-      g_debug ("(II) reducing bytes_requested to avoid reading past EOF\n");
-      bytes_requested = handle->size - offset;
-    }
-
     unsigned char *temp;
     int ret = LIBMTP_GetPartialObject (G_VFS_BACKEND_MTP (backend)->device, id, offset,
                                        bytes_requested, &temp, &actual);
@@ -2491,7 +2464,6 @@ do_read (GVfsBackend *backend,
     memcpy (buffer, bytes->data + offset, actual);
   }
 
- finished:
   handle->offset = offset + actual;
   g_vfs_job_read_set_size (job, actual);
   g_vfs_job_succeeded (G_VFS_JOB (job));
